describe('DateTimeWidget', function(){

  var input;

  beforeEach(function(){
    input = $('<input>', {
      id:'#input'
    })
  })

  afterEach(function(){
    input.remove()
  })

  describe("Plugin definition", function(){

    it.only('should be defined on jquery object', function(){
      expect(input.dtwidget).to.exist
    })

    it('should return element', function(){
      expect(input.dtwidget()).to.equal(input)
    })

    it('should store instance in data object', function(){
      input.dtwidget()
      expect(input.data('dtwidget')).to.exist
    })

  })

  describe("Date and Format parsing", function(){

    it('should parse format from options', function(){
      input.dtwidget({
        format: "dd.MM.yyyy HH:mm"
      })

      instance = input.data('dtwidget')
      var result = instance.parseFormat()

      expect(result.order).to.eql(['d', 'M', 'y', 'H', 'm'])

      var segs = result.segments

      expect(segs.d).to.eql({
        pos: [0, 2]
        , _next: segs.M
        , char: 'd'
      })

      expect(segs.M).to.eql({
        pos: [3, 5]
        , _prev: segs.d
        , _next: segs.y
        , char: 'M'
      })

      expect(segs.y).to.eql({
        pos: [6, 10]
        , _prev: segs.M
        , _next: segs.H
        , char: 'y'
      })

      expect(segs.H).to.eql({
        pos: [11, 13]
        , _prev: segs.y
        , _next: segs.m
        , char: 'H'
      })

      expect(segs.m).to.eql({
        pos: [14, 16]
        , _prev: segs.H
        , char: 'm'
      })

    })

    it('should be able to step through date segments', function(){

      input.dtwidget({
        format: "dd.MM.yyyy HH:mm"
      })

      instance = input.data('dtwidget')

      expect(instance.current).to.be.undefined
      instance.current = instance.first()
      expect(instance.current.char).to.equal('d')
      instance.next()
      expect(instance.current.char).to.equal('M')
      instance.next()
      expect(instance.current.char).to.equal('y')
      instance.next()
      expect(instance.current.char).to.equal('H')
      instance.next()
      expect(instance.current.char).to.equal('m')
      // go back to first
      instance.next()
      expect(instance.current.char).to.equal('d')

      // now backwards start at last item
      instance.current = instance.last()
      expect(instance.current.char).to.equal('m')
      instance.prev()
      expect(instance.current.char).to.equal('H')
      instance.prev()
      expect(instance.current.char).to.equal('y')
      instance.prev()
      expect(instance.current.char).to.equal('M')
      instance.prev()
      expect(instance.current.char).to.equal('d')
      // go back to last
      instance.prev()
      expect(instance.current.char).to.equal('m')


    })

    it('should output the formatted date', function(){
      input.dtwidget({
        date: new Date(2012, 11, 2, 12, 30)
        , format: "dd.MM.yyyy HH:mm"
      })

      instance = input.data('dtwidget')
      expect(instance.formatted()).to.equal('02.12.2012 12:30')

      expect(instance.formatted('yyyy-MM-dd HH:mm')).to.equal('2012-12-02 12:30')

    })

    it('should convert value to date with format', function(){

      input = $('<input>', {
        id:'#input'
        , value: '02.11.2012 12:30'
      })

      input.dtwidget({
        format: "dd.MM.yyyy HH:mm"
      })

      instance = input.data('dtwidget')
      expect(instance.formatted()).to.equal('02.11.2012 12:30')
    })

  })

  describe("Increment and decrement Datetime", function(){

    var input, instance

    beforeEach(function(){
      input = $('<input>', {
        id:'#input'
        , value: '02.02.2012 12:30'
      })

      input.dtwidget({
        format: "dd.MM.yyyy HH:mm"
      })

      instance = input.data('dtwidget')
    })

    afterEach(function(){
      input.remove()
    })

    it("should be able to increment/decrement days", function(){
      instance.addDays(2)
      expect(input.val()).to.equal('04.02.2012 12:30')

      instance.addDays(-3)
      expect(input.val()).to.equal('01.02.2012 12:30')

      // increment to next month including leap year handling
      instance.addDays(29)
      expect(input.val()).to.equal('01.03.2012 12:30')

      // decrement to last month
      instance.addDays(-1)
      expect(input.val()).to.equal('29.02.2012 12:30')

    })

    it("should be able to increment/decrement months", function(){

      instance.addMonths(4)
      expect(input.val()).to.equal('02.06.2012 12:30')

      instance.addMonths(-1)
      expect(input.val()).to.equal('02.05.2012 12:30')

      // increment to next year
      instance.addMonths(12)
      expect(input.val()).to.equal('02.05.2013 12:30')

      // decrement to last month
      instance.addMonths(-5)
      expect(input.val()).to.equal('02.12.2012 12:30')

    })

    it("should be able to increment/decrement year", function(){

      instance.addYears(10)
      expect(input.val()).to.equal('02.02.2022 12:30')

      instance.addYears(-2)
      expect(input.val()).to.equal('02.02.2020 12:30')

    })

    it("should be able to increment/decrement hours", function(){

      instance.addHours(2)
      expect(input.val()).to.equal('02.02.2012 14:30')

      instance.addHours(-1)
      expect(input.val()).to.equal('02.02.2012 13:30')

      // increment to next day
      instance.addHours(12)
      expect(input.val()).to.equal('03.02.2012 01:30')

      // decrement to last day
      instance.addHours(-2)
      expect(input.val()).to.equal('02.02.2012 23:30')

    })

    it("should be able to increment/decrement minutes", function(){

      instance.addMinutes(12)
      expect(input.val()).to.equal('02.02.2012 12:42')

      instance.addMinutes(-16)
      expect(input.val()).to.equal('02.02.2012 12:26')

      // increment to next hour
      instance.addMinutes(34)
      expect(input.val()).to.equal('02.02.2012 13:00')

      // decrement to last hour
      instance.addMinutes(-1)
      expect(input.val()).to.equal('02.02.2012 12:59')

    })

  })

  describe("User interaction", function(){

    var input, instance

    beforeEach(function(){
      input = $('<input>', {
        id:'#input'
        , value: '02.02.2012 12:30'
      })

      input.dtwidget({
        format: "dd.MM.yyyy HH:mm"
      })

      instance = input.data('dtwidget')
    })

    afterEach(function(){
      input.remove()
    })

    // Helper function for selections

    function select(el, start, end){
      el.selectionStart = start
      el.selectionEnd = end || start
      el.focus
      $(el).trigger('click')
    }

    function selection(el){
      return [el.selectionStart, el.selectionEnd]
    }

    function triggerKeydown(el, char){
      el.trigger({ type: 'keydown', which: char })
    }

    it("should be able to select a segment", function(){

      var el = instance.element

      // *click* on days
      select(el, 1)
      expect(instance.current.char).to.equal('d')
      expect(selection(el)).to.eql([0, 2]) // whole day segment should be selected now

      // *click* on month
      select(el, 3)
      expect(instance.current.char).to.equal('M')
      expect(selection(el)).to.eql([3, 5])

      // *click* on year
      select(el, 10)
      expect(instance.current.char).to.equal('y')
      expect(selection(el)).to.eql([6, 10])

      // *click* on hour
      select(el, 12)
      expect(instance.current.char).to.equal('H')
      expect(selection(el)).to.eql([11, 13])

      // *click* on minute
      select(el, 14)
      expect(instance.current.char).to.equal('m')
      expect(selection(el)).to.eql([14, 16])


    })

    it("should increase/decreare value with key up/down", function(){

      var el = instance.element

      // select day
      // select(el, 1)
      // triggerKeydown(input, 40) // arrow down
      // expect(input.val()).to.equal('01.02.2012 12:30')
      // triggerKeydown(input, 40) // arrow down
      // expect(input.val()).to.equal('31.01.2012 12:30')

      // triggerKeydown(input, 38) // arrow up
      //expect(input.val()).to.equal('01.02.2012 12:30')
    })

    // TODO
    // restore selection on render
    // cycle with keys
    // skip chars
    // esc
    // direct input

  })

})


